/***************************************************************************
 *
 * PROJECT: The Dark Mod
 * $Revision: 4761 $
 * $Date: 2011-04-09 11:30:34 +0200 (Sa, 09 Apr 2011) $
 * $Author: greebo $
 *
 ***************************************************************************/

#include "../idlib/precompiled.h"
#pragma hdrstop

static bool init_version = FileVersionList("$Id: ShopItem.cpp 4761 2011-04-09 09:30:34Z greebo $", init_version);

#include "ShopItem.h"

CShopItem::CShopItem() :
	id(""),
	name(""),
	description(""),
	cost(0),
	image(""),
	count(0),
	persistent(false),
	canDrop(false),
	dropped(0),			// tels (#2567) remember how many we dropped	
	stackable(false)	// grayman (#2376)
{}

CShopItem::CShopItem(const idStr& _id, const idStr& _name, const idStr& _description,
					 int _cost, const idStr& _image, int _count, bool _persistent, bool _canDrop, bool _stackable) : // grayman (#2376)
	id(_id),
	name(_name),
	description(_description),
	cost(_cost),
	image(_image),
	count(_count),
	persistent(_persistent),
	canDrop(_canDrop),
	dropped(0),
	stackable(_stackable) // grayman (#2376)
{}

CShopItem::CShopItem(const CShopItem& item, int _count, int _cost, bool _persistent) :
	id(item.id),
	name(item.name),
	description(item.description),
	cost(_cost == 0 ? item.cost : _cost),
	image(item.image),
	count(_count),
	persistent(_persistent == false ? item.persistent : _persistent),
	canDrop(item.canDrop),
	dropped(0),
	classNames(item.classNames),
	stackable(item.stackable) // grayman (#2376)
{}

const idStr& CShopItem::GetID() const {
	return this->id;
}

const idStr& CShopItem::GetName() const {
	return this->name;
}

const idStr& CShopItem::GetDescription() const {
	return this->description;
}

const idStringList& CShopItem::GetClassnames() const
{
	return classNames;
}

void CShopItem::AddClassname(const idStr& className)
{
	classNames.AddUnique(className);
}

const idStr& CShopItem::GetImage() const {
	return this->image;
}

int CShopItem::GetCost() {
	return this->cost;
}

int CShopItem::GetCount() {
	return this->count;
}

int CShopItem::GetDroppedCount() {
	return this->dropped;
}

bool CShopItem::GetPersistent() {
	return this->persistent;
}

bool CShopItem::GetCanDrop() {
	return this->canDrop;
}

void CShopItem::SetCanDrop(bool canDrop) {
	this->canDrop = canDrop;
}

// tels (#2567)
void CShopItem::Drop(void) {
	if (this->canDrop && this->count > 0)
	{
		this->dropped = this->count;
		this->count = 0;
	}
}

// tels (#2567)
void CShopItem::Undrop(void) {
	if (this->canDrop && this->dropped > 0)
	{
		this->count = this->dropped;
		this->dropped = 0;
	}
}

// grayman (#2376) - add stackable methods
bool CShopItem::GetStackable() {
	return this->stackable;
}

void CShopItem::SetStackable(bool stackable) {
	this->stackable = stackable;
}

void CShopItem::ChangeCount(int amount) {
	this->count += amount;
}

void CShopItem::Save(idSaveGame *savefile) const
{
	savefile->WriteString(id);
	savefile->WriteString(name);
	savefile->WriteString(description);

	savefile->WriteInt(cost);
	savefile->WriteString(image);
	savefile->WriteInt(count);
	savefile->WriteInt(dropped);
	savefile->WriteBool(persistent);
	savefile->WriteBool(canDrop);

	savefile->WriteInt(classNames.Num());
	for (int i = 0; i < classNames.Num(); ++i)
	{
		savefile->WriteString(classNames[i]);
	}
	savefile->WriteBool(stackable); // grayman (#2376)
}

void CShopItem::Restore(idRestoreGame *savefile)
{
	savefile->ReadString(id);
	savefile->ReadString(name);
	savefile->ReadString(description);

	savefile->ReadInt(cost);
	savefile->ReadString(image);
	savefile->ReadInt(count);
	savefile->ReadInt(dropped);
	savefile->ReadBool(persistent);
	savefile->ReadBool(canDrop);

	int temp;
	savefile->ReadInt(temp);
	classNames.SetNum(temp);
	for (int i = 0; i < temp; ++i)
	{
		savefile->ReadString(classNames[i]);
	}
	savefile->ReadBool(stackable); // grayman (#2376)
}
